# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/mlflow/v1alpha1/cluster_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.msp.mlflow.v1alpha1 import cluster_pb2 as nebius_dot_msp_dot_mlflow_dot_v1alpha1_dot_cluster__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1alpha1 import operation_pb2 as nebius_dot_common_dot_v1alpha1_dot_operation__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0nebius/msp/mlflow/v1alpha1/cluster_service.proto\x12\x1anebius.msp.mlflow.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a(nebius/msp/mlflow/v1alpha1/cluster.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a&nebius/common/v1alpha1/operation.proto\x1a\x18nebius/annotations.proto\"+\n\x11GetClusterRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"Z\n\x17GetClusterByNameRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04name\"v\n\x13ListClustersRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"\x81\x01\n\x14ListClustersResponse\x12\x41\n\x05items\x18\x01 \x03(\x0b\x32#.nebius.msp.mlflow.v1alpha1.ClusterB\x06\xbaH\x03\xc8\x01\x01R\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\xd3\x02\n\x14\x43reateClusterRequest\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x43\n\x04spec\x18\x02 \x01(\x0b\x32\'.nebius.msp.mlflow.v1alpha1.ClusterSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec:\xad\x01\xbaH\xa9\x01\x1aZ\n\x12metadata.parent_id\x12&metadata.parent_id should not be empty\x1a\x1chas(this.metadata.parent_id)\x1aK\n\rmetadata.name\x12!metadata.name should not be empty\x1a\x17has(this.metadata.name)\".\n\x14\x44\x65leteClusterRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id2\x8a\x04\n\x0e\x43lusterService\x12Y\n\x03Get\x12-.nebius.msp.mlflow.v1alpha1.GetClusterRequest\x1a#.nebius.msp.mlflow.v1alpha1.Cluster\x12\x65\n\tGetByName\x12\x33.nebius.msp.mlflow.v1alpha1.GetClusterByNameRequest\x1a#.nebius.msp.mlflow.v1alpha1.Cluster\x12i\n\x04List\x12/.nebius.msp.mlflow.v1alpha1.ListClustersRequest\x1a\x30.nebius.msp.mlflow.v1alpha1.ListClustersResponse\x12]\n\x06\x43reate\x12\x30.nebius.msp.mlflow.v1alpha1.CreateClusterRequest\x1a!.nebius.common.v1alpha1.Operation\x12]\n\x06\x44\x65lete\x12\x30.nebius.msp.mlflow.v1alpha1.DeleteClusterRequest\x1a!.nebius.common.v1alpha1.Operation\x1a\r\xbaJ\nmlflow.mspBt\n!ai.nebius.pub.msp.mlflow.v1alpha1B\x13\x43lusterServiceProtoP\x01Z8github.com/nebius/gosdk/proto/nebius/msp/mlflow/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.mlflow.v1alpha1.cluster_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n!ai.nebius.pub.msp.mlflow.v1alpha1B\023ClusterServiceProtoP\001Z8github.com/nebius/gosdk/proto/nebius/msp/mlflow/v1alpha1'
  _GETCLUSTERREQUEST.fields_by_name['id']._options = None
  _GETCLUSTERREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _GETCLUSTERBYNAMEREQUEST.fields_by_name['parent_id']._options = None
  _GETCLUSTERBYNAMEREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _GETCLUSTERBYNAMEREQUEST.fields_by_name['name']._options = None
  _GETCLUSTERBYNAMEREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _LISTCLUSTERSREQUEST.fields_by_name['parent_id']._options = None
  _LISTCLUSTERSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _LISTCLUSTERSRESPONSE.fields_by_name['items']._options = None
  _LISTCLUSTERSRESPONSE.fields_by_name['items']._serialized_options = b'\272H\003\310\001\001'
  _CREATECLUSTERREQUEST.fields_by_name['metadata']._options = None
  _CREATECLUSTERREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _CREATECLUSTERREQUEST.fields_by_name['spec']._options = None
  _CREATECLUSTERREQUEST.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _CREATECLUSTERREQUEST._options = None
  _CREATECLUSTERREQUEST._serialized_options = b'\272H\251\001\032Z\n\022metadata.parent_id\022&metadata.parent_id should not be empty\032\034has(this.metadata.parent_id)\032K\n\rmetadata.name\022!metadata.name should not be empty\032\027has(this.metadata.name)'
  _DELETECLUSTERREQUEST.fields_by_name['id']._options = None
  _DELETECLUSTERREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _CLUSTERSERVICE._options = None
  _CLUSTERSERVICE._serialized_options = b'\272J\nmlflow.msp'
  _globals['_GETCLUSTERREQUEST']._serialized_start=250
  _globals['_GETCLUSTERREQUEST']._serialized_end=293
  _globals['_GETCLUSTERBYNAMEREQUEST']._serialized_start=295
  _globals['_GETCLUSTERBYNAMEREQUEST']._serialized_end=385
  _globals['_LISTCLUSTERSREQUEST']._serialized_start=387
  _globals['_LISTCLUSTERSREQUEST']._serialized_end=505
  _globals['_LISTCLUSTERSRESPONSE']._serialized_start=508
  _globals['_LISTCLUSTERSRESPONSE']._serialized_end=637
  _globals['_CREATECLUSTERREQUEST']._serialized_start=640
  _globals['_CREATECLUSTERREQUEST']._serialized_end=979
  _globals['_DELETECLUSTERREQUEST']._serialized_start=981
  _globals['_DELETECLUSTERREQUEST']._serialized_end=1027
  _globals['_CLUSTERSERVICE']._serialized_start=1030
  _globals['_CLUSTERSERVICE']._serialized_end=1552
# @@protoc_insertion_point(module_scope)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Cluster(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: ClusterSpec
    status: ClusterStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[ClusterSpec, _Mapping]] = ..., status: _Optional[_Union[ClusterStatus, _Mapping]] = ...) -> None: ...

class ClusterSpec(_message.Message):
    __slots__ = ["control_plane", "kube_network"]
    CONTROL_PLANE_FIELD_NUMBER: _ClassVar[int]
    KUBE_NETWORK_FIELD_NUMBER: _ClassVar[int]
    control_plane: ControlPlaneSpec
    kube_network: KubeNetworkSpec
    def __init__(self, control_plane: _Optional[_Union[ControlPlaneSpec, _Mapping]] = ..., kube_network: _Optional[_Union[KubeNetworkSpec, _Mapping]] = ...) -> None: ...

class ControlPlaneSpec(_message.Message):
    __slots__ = ["version", "subnet_id", "endpoints", "etcd_cluster_size"]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    SUBNET_ID_FIELD_NUMBER: _ClassVar[int]
    ENDPOINTS_FIELD_NUMBER: _ClassVar[int]
    ETCD_CLUSTER_SIZE_FIELD_NUMBER: _ClassVar[int]
    version: str
    subnet_id: str
    endpoints: ControlPlaneEndpointsSpec
    etcd_cluster_size: int
    def __init__(self, version: _Optional[str] = ..., subnet_id: _Optional[str] = ..., endpoints: _Optional[_Union[ControlPlaneEndpointsSpec, _Mapping]] = ..., etcd_cluster_size: _Optional[int] = ...) -> None: ...

class ControlPlaneEndpointsSpec(_message.Message):
    __slots__ = ["public_endpoint"]
    PUBLIC_ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    public_endpoint: PublicEndpointSpec
    def __init__(self, public_endpoint: _Optional[_Union[PublicEndpointSpec, _Mapping]] = ...) -> None: ...

class PublicEndpointSpec(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class KubeNetworkSpec(_message.Message):
    __slots__ = ["service_cidrs"]
    SERVICE_CIDRS_FIELD_NUMBER: _ClassVar[int]
    service_cidrs: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, service_cidrs: _Optional[_Iterable[str]] = ...) -> None: ...

class ClusterStatus(_message.Message):
    __slots__ = ["state", "control_plane", "reconciling"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[ClusterStatus.State]
        PROVISIONING: _ClassVar[ClusterStatus.State]
        RUNNING: _ClassVar[ClusterStatus.State]
        DELETING: _ClassVar[ClusterStatus.State]
    STATE_UNSPECIFIED: ClusterStatus.State
    PROVISIONING: ClusterStatus.State
    RUNNING: ClusterStatus.State
    DELETING: ClusterStatus.State
    STATE_FIELD_NUMBER: _ClassVar[int]
    CONTROL_PLANE_FIELD_NUMBER: _ClassVar[int]
    RECONCILING_FIELD_NUMBER: _ClassVar[int]
    state: ClusterStatus.State
    control_plane: ControlPlaneStatus
    reconciling: bool
    def __init__(self, state: _Optional[_Union[ClusterStatus.State, str]] = ..., control_plane: _Optional[_Union[ControlPlaneStatus, _Mapping]] = ..., reconciling: bool = ...) -> None: ...

class ControlPlaneStatus(_message.Message):
    __slots__ = ["version", "endpoints", "etcd_cluster_size", "auth"]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    ENDPOINTS_FIELD_NUMBER: _ClassVar[int]
    ETCD_CLUSTER_SIZE_FIELD_NUMBER: _ClassVar[int]
    AUTH_FIELD_NUMBER: _ClassVar[int]
    version: str
    endpoints: ControlPlaneStatusEndpoints
    etcd_cluster_size: int
    auth: ControlPlaneStatusAuth
    def __init__(self, version: _Optional[str] = ..., endpoints: _Optional[_Union[ControlPlaneStatusEndpoints, _Mapping]] = ..., etcd_cluster_size: _Optional[int] = ..., auth: _Optional[_Union[ControlPlaneStatusAuth, _Mapping]] = ...) -> None: ...

class ControlPlaneStatusEndpoints(_message.Message):
    __slots__ = ["public_endpoint", "private_endpoint"]
    PUBLIC_ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    PRIVATE_ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    public_endpoint: str
    private_endpoint: str
    def __init__(self, public_endpoint: _Optional[str] = ..., private_endpoint: _Optional[str] = ...) -> None: ...

class ControlPlaneStatusAuth(_message.Message):
    __slots__ = ["cluster_ca_certificate"]
    CLUSTER_CA_CERTIFICATE_FIELD_NUMBER: _ClassVar[int]
    cluster_ca_certificate: str
    def __init__(self, cluster_ca_certificate: _Optional[str] = ...) -> None: ...



from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Platform(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: PlatformSpec
    status: PlatformStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[PlatformSpec, _Mapping]] = ..., status: _Optional[_Union[PlatformStatus, _Mapping]] = ...) -> None: ...

class PlatformSpec(_message.Message):
    __slots__ = ["presets", "gpu_count_quota_type", "human_readable_name", "allow_preset_change", "short_human_readable_name", "gpu_memory_gibibytes"]
    PRESETS_FIELD_NUMBER: _ClassVar[int]
    GPU_COUNT_QUOTA_TYPE_FIELD_NUMBER: _ClassVar[int]
    HUMAN_READABLE_NAME_FIELD_NUMBER: _ClassVar[int]
    ALLOW_PRESET_CHANGE_FIELD_NUMBER: _ClassVar[int]
    SHORT_HUMAN_READABLE_NAME_FIELD_NUMBER: _ClassVar[int]
    GPU_MEMORY_GIBIBYTES_FIELD_NUMBER: _ClassVar[int]
    presets: _containers.RepeatedCompositeFieldContainer[Preset]
    gpu_count_quota_type: str
    human_readable_name: str
    allow_preset_change: bool
    short_human_readable_name: str
    gpu_memory_gibibytes: int
    def __init__(self, presets: _Optional[_Iterable[_Union[Preset, _Mapping]]] = ..., gpu_count_quota_type: _Optional[str] = ..., human_readable_name: _Optional[str] = ..., allow_preset_change: bool = ..., short_human_readable_name: _Optional[str] = ..., gpu_memory_gibibytes: _Optional[int] = ...) -> None: ...

class Preset(_message.Message):
    __slots__ = ["name", "resources", "allow_gpu_clustering"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    RESOURCES_FIELD_NUMBER: _ClassVar[int]
    ALLOW_GPU_CLUSTERING_FIELD_NUMBER: _ClassVar[int]
    name: str
    resources: PresetResources
    allow_gpu_clustering: bool
    def __init__(self, name: _Optional[str] = ..., resources: _Optional[_Union[PresetResources, _Mapping]] = ..., allow_gpu_clustering: bool = ...) -> None: ...

class PresetResources(_message.Message):
    __slots__ = ["vcpu_count", "memory_gibibytes", "gpu_count", "gpu_memory_gibibytes"]
    VCPU_COUNT_FIELD_NUMBER: _ClassVar[int]
    MEMORY_GIBIBYTES_FIELD_NUMBER: _ClassVar[int]
    GPU_COUNT_FIELD_NUMBER: _ClassVar[int]
    GPU_MEMORY_GIBIBYTES_FIELD_NUMBER: _ClassVar[int]
    vcpu_count: int
    memory_gibibytes: int
    gpu_count: int
    gpu_memory_gibibytes: int
    def __init__(self, vcpu_count: _Optional[int] = ..., memory_gibibytes: _Optional[int] = ..., gpu_count: _Optional[int] = ..., gpu_memory_gibibytes: _Optional[int] = ...) -> None: ...

class PlatformStatus(_message.Message):
    __slots__ = ["allowed_for_preemptibles"]
    ALLOWED_FOR_PREEMPTIBLES_FIELD_NUMBER: _ClassVar[int]
    allowed_for_preemptibles: bool
    def __init__(self, allowed_for_preemptibles: bool = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/auth_public_key_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.iam.v1 import auth_public_key_pb2 as nebius_dot_iam_dot_v1_dot_auth__public__key__pb2
from nebius.api.nebius.iam.v1 import access_pb2 as nebius_dot_iam_dot_v1_dot_access__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+nebius/iam/v1/auth_public_key_service.proto\x12\rnebius.iam.v1\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a#nebius/iam/v1/auth_public_key.proto\x1a\x1anebius/iam/v1/access.proto\"\x92\x01\n\x1a\x43reateAuthPublicKeyRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x34\n\x04spec\x18\x02 \x01(\x0b\x32 .nebius.iam.v1.AuthPublicKeySpecR\x04spec\")\n\x17GetAuthPublicKeyRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\x9e\x01\n\x18ListAuthPublicKeyRequest\x12\x1b\n\tparent_id\x18\x01 \x01(\tR\x08parentId\x12 \n\tpage_size\x18\x02 \x01(\x03H\x00R\x08pageSize\x88\x01\x01\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12\x16\n\x06\x66ilter\x18\x04 \x01(\tR\x06\x66ilterB\x0c\n\n_page_size\"\xa9\x01\n!ListAuthPublicKeyByAccountRequest\x12\x30\n\x07\x61\x63\x63ount\x18\x01 \x01(\x0b\x32\x16.nebius.iam.v1.AccountR\x07\x61\x63\x63ount\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12\x16\n\x06\x66ilter\x18\x04 \x01(\tR\x06\x66ilter\"\x92\x01\n\x1aUpdateAuthPublicKeyRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x34\n\x04spec\x18\x02 \x01(\x0b\x32 .nebius.iam.v1.AuthPublicKeySpecR\x04spec\".\n\x1c\x41\x63tivateAuthPublicKeyRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"0\n\x1e\x44\x65\x61\x63tivateAuthPublicKeyRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\",\n\x1a\x44\x65leteAuthPublicKeyRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"w\n\x19ListAuthPublicKeyResponse\x12\x32\n\x05items\x18\x01 \x03(\x0b\x32\x1c.nebius.iam.v1.AuthPublicKeyR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xdd\x05\n\x14\x41uthPublicKeyService\x12P\n\x06\x43reate\x12).nebius.iam.v1.CreateAuthPublicKeyRequest\x1a\x1b.nebius.common.v1.Operation\x12K\n\x03Get\x12&.nebius.iam.v1.GetAuthPublicKeyRequest\x1a\x1c.nebius.iam.v1.AuthPublicKey\x12Y\n\x04List\x12\'.nebius.iam.v1.ListAuthPublicKeyRequest\x1a(.nebius.iam.v1.ListAuthPublicKeyResponse\x12k\n\rListByAccount\x12\x30.nebius.iam.v1.ListAuthPublicKeyByAccountRequest\x1a(.nebius.iam.v1.ListAuthPublicKeyResponse\x12P\n\x06Update\x12).nebius.iam.v1.UpdateAuthPublicKeyRequest\x1a\x1b.nebius.common.v1.Operation\x12T\n\x08\x41\x63tivate\x12+.nebius.iam.v1.ActivateAuthPublicKeyRequest\x1a\x1b.nebius.common.v1.Operation\x12X\n\nDeactivate\x12-.nebius.iam.v1.DeactivateAuthPublicKeyRequest\x1a\x1b.nebius.common.v1.Operation\x12P\n\x06\x44\x65lete\x12).nebius.iam.v1.DeleteAuthPublicKeyRequest\x1a\x1b.nebius.common.v1.Operation\x1a\n\xbaJ\x07\x63pl.iamB`\n\x14\x61i.nebius.pub.iam.v1B\x19\x41uthPublicKeyServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.auth_public_key_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\031AuthPublicKeyServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _AUTHPUBLICKEYSERVICE._options = None
  _AUTHPUBLICKEYSERVICE._serialized_options = b'\272J\007cpl.iam'
  _globals['_CREATEAUTHPUBLICKEYREQUEST']._serialized_start=221
  _globals['_CREATEAUTHPUBLICKEYREQUEST']._serialized_end=367
  _globals['_GETAUTHPUBLICKEYREQUEST']._serialized_start=369
  _globals['_GETAUTHPUBLICKEYREQUEST']._serialized_end=410
  _globals['_LISTAUTHPUBLICKEYREQUEST']._serialized_start=413
  _globals['_LISTAUTHPUBLICKEYREQUEST']._serialized_end=571
  _globals['_LISTAUTHPUBLICKEYBYACCOUNTREQUEST']._serialized_start=574
  _globals['_LISTAUTHPUBLICKEYBYACCOUNTREQUEST']._serialized_end=743
  _globals['_UPDATEAUTHPUBLICKEYREQUEST']._serialized_start=746
  _globals['_UPDATEAUTHPUBLICKEYREQUEST']._serialized_end=892
  _globals['_ACTIVATEAUTHPUBLICKEYREQUEST']._serialized_start=894
  _globals['_ACTIVATEAUTHPUBLICKEYREQUEST']._serialized_end=940
  _globals['_DEACTIVATEAUTHPUBLICKEYREQUEST']._serialized_start=942
  _globals['_DEACTIVATEAUTHPUBLICKEYREQUEST']._serialized_end=990
  _globals['_DELETEAUTHPUBLICKEYREQUEST']._serialized_start=992
  _globals['_DELETEAUTHPUBLICKEYREQUEST']._serialized_end=1036
  _globals['_LISTAUTHPUBLICKEYRESPONSE']._serialized_start=1038
  _globals['_LISTAUTHPUBLICKEYRESPONSE']._serialized_end=1157
  _globals['_AUTHPUBLICKEYSERVICE']._serialized_start=1160
  _globals['_AUTHPUBLICKEYSERVICE']._serialized_end=1893
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/compute/v1/operation_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)nebius/compute/v1/operation_service.proto\x12\x11nebius.compute.v1\x1a\x1b\x62uf/validate/validate.proto\"\x80\x01\n\x1dListOperationsByParentRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageTokenBd\n\x18\x61i.nebius.pub.compute.v1B\x15OperationServiceProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/compute/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.compute.v1.operation_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.compute.v1B\025OperationServiceProtoP\001Z/github.com/nebius/gosdk/proto/nebius/compute/v1'
  _LISTOPERATIONSBYPARENTREQUEST.fields_by_name['parent_id']._options = None
  _LISTOPERATIONSBYPARENTREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _globals['_LISTOPERATIONSBYPARENTREQUEST']._serialized_start=94
  _globals['_LISTOPERATIONSBYPARENTREQUEST']._serialized_end=222
# @@protoc_insertion_point(module_scope)



from collections.abc import Iterable, Mapping
from json import dumps, loads
from re import compile
from typing import overload

_simple_string_pattern = compile("^[a-zA-Z0-9_]+$")


class Error(Exception):
    pass


class MarshalError(Error):
    pass


class FieldKey(str):
    @classmethod
    def unmarshal(cls, marshaled_key: str) -> "FieldKey":
        if marshaled_key.startswith('"'):
            return cls(loads(marshaled_key))
        if _simple_string_pattern.match(marshaled_key):
            return cls(marshaled_key)
        raise MarshalError("malformed FieldKey string")

    def marshal(self) -> str:
        if _simple_string_pattern.match(self):
            return str(self)
        else:
            return dumps(str(self))


class FieldPath(list[FieldKey]):
    @overload
    def __init__(self) -> None: ...

    @overload
    def __init__(self, base: Iterable[FieldKey | str]) -> None: ...

    def __init__(self, base: Iterable[FieldKey | str] | None) -> None:  # type: ignore
        super().__init__()
        if base is not None:
            if not isinstance(base, Iterable):  # type: ignore[unused-ignore]
                raise ValueError(f"base should be iterable, got {type(base)}")
            for v in base:
                if isinstance(v, str):  # type: ignore[unused-ignore]
                    v = FieldKey(v)
                if not isinstance(v, FieldKey):  # type: ignore[unused-ignore]
                    raise ValueError(
                        "base contents should be FieldKey or str, got " f"{type(v)}"
                    )
                self.append(v)

    def parent(self) -> "FieldPath|None":
        if len(self) == 0:
            return None
        return FieldPath(self[:-1].copy())

    def copy(self) -> "FieldPath":
        return FieldPath(super().copy())

    def to_mask(self) -> "Mask":
        ret = Mask()
        cur = ret
        for v in self:
            nxt = Mask()
            cur.field_parts[v] = nxt
            cur = nxt
        return ret

    @staticmethod
    def _matches_reset_mask(
        fp: "FieldPath|list[FieldKey]", mask: "Mask|None"
    ) -> tuple[bool, bool]:
        if not isinstance(mask, Mask):
            return False, False
        if len(fp) == 0:
            return True, mask.is_empty()
        key, rest = fp[0], fp[1:]
        has_match = False
        is_final = False
        if mask.any is not None:
            has_match, is_final = FieldPath._matches_reset_mask(rest, mask.any)
        if key in mask.field_parts:
            k_match, k_final = FieldPath._matches_reset_mask(
                rest, mask.field_parts[key]
            )
            has_match |= k_match
            if k_match:
                is_final |= k_final
        return has_match, is_final

    @staticmethod
    def _matches_select_mask(
        fp: "FieldPath|list[FieldKey]", mask: "Mask|None"
    ) -> tuple[bool, bool]:
        if mask is None or mask.is_empty():
            return True, len(fp) != 0
        if len(fp) == 0:
            return True, False
        key, rest = fp[0], fp[1:]
        has_match = False
        is_inner = False
        if mask.any is not None:
            has_match, is_inner = FieldPath._matches_select_mask(rest, mask.any)
        if key in mask.field_parts:
            k_match, k_final = FieldPath._matches_select_mask(
                rest, mask.field_parts[key]
            )
            has_match |= k_match
            if k_match:
                is_inner |= k_final
        return has_match, is_inner

    def matches_reset_mask(self, mask: "Mask|None") -> bool:
        ret, _ = FieldPath._matches_reset_mask(self, mask)
        return ret

    def matches_reset_mask_final(self, mask: "Mask|None") -> bool:
        ret, fin = FieldPath._matches_reset_mask(self, mask)
        return ret and fin

    def matches_select_mask(self, mask: "Mask|None") -> bool:
        ret, _ = FieldPath._matches_select_mask(self, mask)
        return ret

    def matches_select_mask_inner(self, mask: "Mask|None") -> tuple[bool, bool]:
        return FieldPath._matches_select_mask(self, mask)

    def marshal(self) -> str:
        return self.to_mask().marshal()

    def __repr__(self) -> str:
        try:
            return f"FieldPath({self.marshal()})"
        except Exception as e:
            return f"FieldPath(not-marshalable {e})"

    @classmethod
    def unmarshal(cls, s: str) -> "FieldPath|None":
        return Mask.unmarshal(s).to_field_path()

    def __iadd__(self, value: "Iterable[FieldKey|str]") -> "FieldPath":  # type: ignore[misc,override]
        for v in value:
            if isinstance(v, str):  # type: ignore[unused-ignore]
                v = FieldKey(v)
            if not isinstance(v, FieldKey):  # type: ignore[unused-ignore]
                raise ValueError(
                    "value contents should be FieldKey or str, got " f"{type(v)}"
                )
            self.append(v)
        return self

    @overload  # type: ignore[override]
    def __add__(self, value: Iterable[FieldKey | str]) -> "FieldPath": ...

    @overload
    def __add__(self, value: "Mask") -> "Mask": ...

    def __add__(self, other: "Iterable[FieldKey|str]|Mask") -> "FieldPath|Mask":  # type: ignore[unused-ignore]
        if isinstance(other, Mask):
            mask = Mask()
            cur = mask
            for i, v in enumerate(self):
                nxt = Mask()
                if i == len(self) - 1:
                    nxt = other
                cur.field_parts[v] = nxt
                cur = nxt
            return mask
        cp = self.copy()
        cp += other
        return cp

    def __eq__(self, value: object) -> bool:
        if not isinstance(value, FieldPath):
            return False
        return super().__eq__(value)

    def is_prefix_of(self, value: "FieldPath") -> bool:
        if not isinstance(value, FieldPath):  # type: ignore[unused-ignore]
            return False
        if len(self) >= len(value):
            return False
        for i, v in enumerate(self):
            if value[i] != v:
                return False
        return True


class Mask:
    def __init__(
        self,
        any: "Mask|None" = None,
        field_parts: Mapping[FieldKey | str, "Mask"] | None = None,
    ) -> None:
        if any is not None and not isinstance(any, Mask):  # type: ignore[unused-ignore]
            raise ValueError(f"any should be Map or None, got {type(any)}")
        self.any: "Mask|None" = any
        self.field_parts = dict[FieldKey, "Mask"]()
        if isinstance(field_parts, Mapping):
            for k, v in field_parts.items():
                if isinstance(k, str):  # type: ignore[unused-ignore]
                    k = FieldKey(k)
                if not isinstance(k, FieldKey):  # type: ignore[unused-ignore]
                    raise ValueError(
                        "field_parts keys should be FieldKey or str, got" f" {type(k)}"
                    )
                if not isinstance(v, Mask):  # type: ignore[unused-ignore]
                    raise ValueError(
                        "field_parts values should be of type Mask, " f"got {type(v)}"
                    )
                self.field_parts[k] = v

    def is_empty(self) -> bool:
        return (self.any is None) and len(self.field_parts) == 0

    def to_field_path(self) -> FieldPath | None:
        if self.any is not None:
            raise Error("wildcard in the mask")
        if len(self.field_parts) > 1:
            raise Error("multiple paths in the mask")
        for k, v in self.field_parts.items():
            inner = v.to_field_path()
            if inner is None:
                return FieldPath([k])
            return FieldPath([k]) + inner
        return None

    def is_field_path(self) -> bool:
        try:
            self.to_field_path()
        except Error:
            return False
        return True

    def __iadd__(self, other: "Mask|FieldPath|None") -> "Mask":
        if isinstance(other, FieldPath):
            other = other.to_mask()
        if other is None or other.is_empty():
            return self
        if self.any is not None:
            self.any += other.any
        elif other.any is not None:
            self.any = other.any.copy()
        for k, v in other.field_parts.items():
            if k in self.field_parts:
                self.field_parts[k] += v
            else:
                self.field_parts[k] = v.copy()
        return self

    def __add__(self, other: "Mask|FieldPath|None") -> "Mask":
        cp = self.copy()
        cp += other
        return cp

    def copy(self) -> "Mask":
        ret = Mask()
        if self.any is not None:
            ret.any = self.any.copy()
        for k, v in self.field_parts.items():
            ret.field_parts[k] = v.copy()
        return ret

    def sub_mask(self, path: FieldPath | FieldKey) -> "Mask|None":
        if isinstance(path, FieldKey):
            if path in self.field_parts:
                if self.any is not None:
                    ret: Mask | None = self.field_parts[path].copy()
                    if ret is None:
                        return ret
                    ret += self.any
                    return ret
                return self.field_parts[path]
            return self.any
        if isinstance(path, FieldPath):  # type: ignore[unused-ignore]
            ret = self
            for s in path:
                if ret is None:
                    return None
                ret = ret.sub_mask(s)
            return ret
        raise Error(f"unexpected path type {type(path)}")

    def __repr__(self) -> str:
        try:
            return f"Mask({self.marshal()})"
        except RecursionError:
            return "Mask(not-marshalable <too deep>)"
        except Exception as e:
            return f"Mask(not-marshalable {e})"

    def __eq__(self, value: object) -> bool:
        try:
            if not isinstance(value, Mask):
                return False
            if self.any != value.any:
                return False
            if len(self.field_parts) != len(value.field_parts):
                return False
            for k, v in self.field_parts.items():
                if k not in value.field_parts or value.field_parts[k] != v:
                    return False
        except RecursionError:
            return False
        return True

    def _marshal(self) -> tuple[int, str]:
        if self.is_empty():
            return 0, ""
        ret = list[str]()

        def append_to_ret(key: str, mask: "Mask") -> None:
            counter, mask_str = mask._marshal()
            if mask_str == "":
                ret.append(key)
            elif counter == 1:
                ret.append(key + "." + mask_str)
            else:
                ret.append(key + ".(" + mask_str + ")")

        if self.any is not None:
            append_to_ret("*", self.any)
        for k, v in self.field_parts.items():
            append_to_ret(k.marshal(), v)
        ret = sorted(ret)
        return len(ret), ",".join(ret)

    def marshal(self) -> str:
        return self._marshal()[1]

    def intersect_reset_mask(self, other: "Mask|None") -> "Mask|None":
        ret = Mask()
        if not isinstance(other, Mask):
            return None
        if self.any is not None:
            ret.any = self.any.intersect_reset_mask(other.any)
            for k, v in other.field_parts.items():
                inner = self.any.intersect_reset_mask(v)
                if inner is not None:
                    ret.field_parts[k] = inner
        if other.any is not None:
            for k, v in self.field_parts.items():
                inner = other.any.intersect_reset_mask(v)
                if inner is not None:
                    if k in ret.field_parts:
                        ret.field_parts[k] += inner
                    else:
                        ret.field_parts[k] = inner
        for k, v in self.field_parts.items():
            if k in other.field_parts:
                inner = v.intersect_reset_mask(other.field_parts[k])
                if inner is not None:
                    if k in ret.field_parts:
                        ret.field_parts[k] += inner
                    else:
                        ret.field_parts[k] = inner
        return ret

    def __and__(self, other: "Mask|None") -> "Mask|None":
        return self.intersect_reset_mask(other)

    def intersect_dumb(self, other: "Mask|None") -> "Mask|None":
        if not isinstance(other, Mask):
            return None
        ret = Mask()
        if self.any is not None:
            ret.any = self.any.intersect_dumb(other.any)
        for k, v in self.field_parts.items():
            if k in other.field_parts:
                inner = v.intersect_dumb(other.field_parts[k])
                if inner is not None:
                    ret.field_parts[k] = inner
        return ret

    def __imul__(self, other: "Mask") -> "Mask":
        if not isinstance(other, Mask):  # type: ignore[unused-ignore]
            raise ValueError(f"argument 2 must be Mask, {type(other)} given")
        ret = self.intersect_dumb(other)
        if not isinstance(ret, Mask):  # type: ignore[unused-ignore]
            raise Error(f"return should have been Mask, got {type(ret)}")
        return ret

    def __mul__(self, other: "Mask") -> "Mask":
        cp = self.copy()
        cp *= other
        return cp

    def subtract_dumb(self, other: "Mask|None") -> None:
        if not isinstance(other, Mask):
            return
        if self.any is not None and other.any is not None:
            self.any.subtract_dumb(other.any)
            if self.any.is_empty():
                self.any = None
        for k in list(self.field_parts.keys()):
            if k in other.field_parts:
                self.field_parts[k].subtract_dumb(other.field_parts[k])
                if self.field_parts[k].is_empty():
                    del self.field_parts[k]

    def __itruediv__(self, other: "Mask") -> "Mask":
        if not isinstance(other, Mask):  # type: ignore[unused-ignore]
            raise ValueError(f"argument 2 must be Mask, {type(other)} given")
        self.subtract_dumb(other)
        return self

    def __truediv__(self, other: "Mask") -> "Mask":
        cp = self.copy()
        cp /= other
        return cp

    def subtract_reset_mask(self, other: "Mask|None") -> None:
        if not isinstance(other, Mask):
            return
        if self.any is not None:
            self.any.subtract_reset_mask(other.any)
            if self.any.is_empty():
                self.any = None
        for k in list(self.field_parts.keys()):
            if other.any is None and k not in other.field_parts:
                continue

            if other.any is not None:
                self.field_parts[k].subtract_reset_mask(other.any)
            if k in other.field_parts:
                self.field_parts[k].subtract_reset_mask(other.field_parts[k])
            if self.field_parts[k].is_empty():
                del self.field_parts[k]

    @classmethod
    def unmarshal(cls, source: str) -> "Mask":
        from .fieldmask_parser import parse

        return parse(source)

    def __isub__(self, other: "Mask|None") -> "Mask":
        self.subtract_reset_mask(other)
        return self

    def __sub__(self, other: "Mask|None") -> "Mask":
        cp = self.copy()
        cp -= other
        return cp



from nebius.api.nebius.mysterybox.v1 import secret_pb2 as _secret_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius.common.v1 import operation_pb2 as _operation_pb2
from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CreateSecretRequest(_message.Message):
    __slots__ = ["metadata", "spec"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: _secret_pb2.SecretSpec
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[_secret_pb2.SecretSpec, _Mapping]] = ...) -> None: ...

class UpdateSecretRequest(_message.Message):
    __slots__ = ["metadata", "spec"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: _secret_pb2.SecretSpec
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[_secret_pb2.SecretSpec, _Mapping]] = ...) -> None: ...

class GetSecretRequest(_message.Message):
    __slots__ = ["id", "show_scheduled_for_deletion"]
    ID_FIELD_NUMBER: _ClassVar[int]
    SHOW_SCHEDULED_FOR_DELETION_FIELD_NUMBER: _ClassVar[int]
    id: str
    show_scheduled_for_deletion: bool
    def __init__(self, id: _Optional[str] = ..., show_scheduled_for_deletion: bool = ...) -> None: ...

class GetSecretByNameRequest(_message.Message):
    __slots__ = ["parent_id", "name"]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    parent_id: str
    name: str
    def __init__(self, parent_id: _Optional[str] = ..., name: _Optional[str] = ...) -> None: ...

class ListSecretsRequest(_message.Message):
    __slots__ = ["parent_id", "page_size", "page_token", "show_scheduled_for_deletion"]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    SHOW_SCHEDULED_FOR_DELETION_FIELD_NUMBER: _ClassVar[int]
    parent_id: str
    page_size: int
    page_token: str
    show_scheduled_for_deletion: bool
    def __init__(self, parent_id: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., show_scheduled_for_deletion: bool = ...) -> None: ...

class ListSecretsResponse(_message.Message):
    __slots__ = ["next_page_token", "items"]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    next_page_token: str
    items: _containers.RepeatedCompositeFieldContainer[_secret_pb2.Secret]
    def __init__(self, next_page_token: _Optional[str] = ..., items: _Optional[_Iterable[_Union[_secret_pb2.Secret, _Mapping]]] = ...) -> None: ...

class DeleteSecretRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class UndeleteSecretRequest(_message.Message):
    __slots__ = ["id", "name"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    def __init__(self, id: _Optional[str] = ..., name: _Optional[str] = ...) -> None: ...



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class RouteTable(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: RouteTableSpec
    status: RouteTableStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[RouteTableSpec, _Mapping]] = ..., status: _Optional[_Union[RouteTableStatus, _Mapping]] = ...) -> None: ...

class RouteTableSpec(_message.Message):
    __slots__ = ["network_id"]
    NETWORK_ID_FIELD_NUMBER: _ClassVar[int]
    network_id: str
    def __init__(self, network_id: _Optional[str] = ...) -> None: ...

class RouteTableStatus(_message.Message):
    __slots__ = ["state", "default", "assignment"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[RouteTableStatus.State]
        READY: _ClassVar[RouteTableStatus.State]
    STATE_UNSPECIFIED: RouteTableStatus.State
    READY: RouteTableStatus.State
    STATE_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_FIELD_NUMBER: _ClassVar[int]
    ASSIGNMENT_FIELD_NUMBER: _ClassVar[int]
    state: RouteTableStatus.State
    default: bool
    assignment: RouteTableAssignment
    def __init__(self, state: _Optional[_Union[RouteTableStatus.State, str]] = ..., default: bool = ..., assignment: _Optional[_Union[RouteTableAssignment, _Mapping]] = ...) -> None: ...

class RouteTableAssignment(_message.Message):
    __slots__ = ["subnets"]
    SUBNETS_FIELD_NUMBER: _ClassVar[int]
    subnets: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, subnets: _Optional[_Iterable[str]] = ...) -> None: ...



import argparse
import ast
import logging
import sys

# Custom types for readability
PrefixMap = dict[str, str]


class Replacement:
    def __init__(
        self,
        start_line: int,
        start_col: int,
        end_line: int | None,
        end_col: int | None,
        replace_to: str,
    ) -> None:
        self.start_line = start_line
        self.start_col = start_col
        self.end_line = end_line
        self.end_col = end_col
        self.replace_to = replace_to


ReplacementPositions = dict[tuple[int, int], Replacement]


class ImportTransformer(ast.NodeVisitor):
    """
    AST visitor that collects replacement positions for imports and identifiers based
    on a prefix map.
    """

    def __init__(self, prefix_map: PrefixMap) -> None:
        self.prefix_map = prefix_map
        self.replacements: ReplacementPositions = {}

    def visit_Import(self, node: ast.Import) -> None:  # noqa: N802
        for alias in node.names:
            new_name = self._replace_prefix(alias.name)
            if new_name != alias.name:
                alias.name = new_name  # Update AST node
                replacement_code = ast.unparse(alias)  # Unparse modified node to code
                replacement = Replacement(
                    alias.lineno,
                    alias.col_offset,
                    alias.end_lineno,
                    alias.end_col_offset,
                    replacement_code,
                )
                self.replacements[(alias.lineno, alias.col_offset)] = replacement

    def visit_ImportFrom(self, node: ast.ImportFrom) -> None:  # noqa: N802
        if node.module:
            new_name = self._replace_prefix(node.module)
            if new_name != node.module:
                node.module = new_name  # Update AST node
                replacement_code = ast.unparse(node)  # Unparse modified node to code
                replacement = Replacement(
                    node.lineno,
                    node.col_offset,
                    node.end_lineno,
                    node.end_col_offset,
                    replacement_code,
                )
                self.replacements[(node.lineno, node.col_offset)] = replacement

    def _replace_prefix(self, name: str) -> str:
        """Replace the prefix for imported names"""
        for old_prefix, new_prefix in self.prefix_map.items():
            if name.startswith(old_prefix + ".") or name == old_prefix:
                return name.replace(old_prefix, new_prefix, 1)
        return name


def parse_prefix_map(prefix_str: list[str]) -> PrefixMap:
    prefix_map: PrefixMap = {}
    for pair in prefix_str:
        if "=" not in pair:
            raise ValueError(f"Invalid prefix format: '{pair}', expected format 'A=B'")
        old, new = pair.split("=", 1)
        prefix_map[old] = new
    return prefix_map


def apply_replacements(code: str, replacements: ReplacementPositions) -> str:
    """Apply replacements to code based on line and column positions."""
    lines = code.splitlines(keepends=True)

    for _, replacement in sorted(replacements.items(), reverse=True):
        start_line = replacement.start_line
        start_col = replacement.start_col
        end_line = replacement.end_line
        end_col = replacement.end_col
        replace_to = replacement.replace_to

        # Handle replacement logic based on line and column range
        if end_line is None or start_line == end_line:
            # If there's no end_line, or it's a single-line replacement
            line = lines[start_line - 1]  # 0-based index
            lines[start_line - 1] = line[:start_col] + replace_to + line[end_col:]
        else:
            # Multi-line replacement
            # Replace the portion on the start line
            line = lines[start_line - 1]
            lines[start_line - 1] = (
                line[:start_col] + replace_to + line[start_col + len(replace_to) :]
            )

            # Replace entire lines between start_line and end_line (exclusive)
            for i in range(start_line, end_line - 1):
                lines[i] = replace_to

            # Replace the portion on the end line
            line = lines[end_line - 1]
            lines[end_line - 1] = (
                line[:start_col] + replace_to + line[start_col + len(replace_to) :]
            )

    return "".join(lines)


def main() -> None:
    parser = argparse.ArgumentParser(
        description="Modify Python file imports while preserving formatting."
    )
    parser.add_argument("--input", type=str, help="Path to the input Python file")
    parser.add_argument("--output", type=str, help="Path to the output Python file")
    parser.add_argument(
        "--prefix",
        type=str,
        nargs="+",
        required=True,
        help="Prefix mappings in format 'A=B'",
    )
    parser.add_argument(
        "--level", type=str, default="info", help="Logging level (default: info)"
    )
    args = parser.parse_args()

    # Configure logging
    logging.basicConfig(
        level=getattr(logging, args.level.upper(), None),
        stream=sys.stderr,
        format="%(asctime)s - %(levelname)s - %(message)s",
    )
    prefix_map = parse_prefix_map(args.prefix)
    logging.info(f"Prefix map: {prefix_map}")

    # Read the input file
    if args.input:
        with open(args.input, "r") as f:
            code = f.read()
    else:
        code = sys.stdin.read()

    # Parse the code into an AST for analysis
    try:
        tree = ast.parse(code)
        logging.info("Parsed input file successfully.")
    except SyntaxError as e:
        logging.error(f"Syntax error in input file: {e}")
        sys.exit(1)

    # Analyze the AST for transformations
    transformer = ImportTransformer(prefix_map)
    transformer.visit(tree)

    # Apply replacements by line and column positions
    modified_code = apply_replacements(code, transformer.replacements)

    # Write the modified code to the output
    if args.output:
        with open(args.output, "w") as f:
            f.write(modified_code)
            logging.info(f"Modified code written to {args.output}.")
    else:
        print(modified_code)


if __name__ == "__main__":
    main()



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Network(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: NetworkSpec
    status: NetworkStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[NetworkSpec, _Mapping]] = ..., status: _Optional[_Union[NetworkStatus, _Mapping]] = ...) -> None: ...

class NetworkSpec(_message.Message):
    __slots__ = ["ipv4_private_pools", "ipv4_public_pools"]
    IPV4_PRIVATE_POOLS_FIELD_NUMBER: _ClassVar[int]
    IPV4_PUBLIC_POOLS_FIELD_NUMBER: _ClassVar[int]
    ipv4_private_pools: IPv4PrivateNetworkPools
    ipv4_public_pools: IPv4PublicNetworkPools
    def __init__(self, ipv4_private_pools: _Optional[_Union[IPv4PrivateNetworkPools, _Mapping]] = ..., ipv4_public_pools: _Optional[_Union[IPv4PublicNetworkPools, _Mapping]] = ...) -> None: ...

class IPv4PrivateNetworkPools(_message.Message):
    __slots__ = ["pools"]
    POOLS_FIELD_NUMBER: _ClassVar[int]
    pools: _containers.RepeatedCompositeFieldContainer[NetworkPool]
    def __init__(self, pools: _Optional[_Iterable[_Union[NetworkPool, _Mapping]]] = ...) -> None: ...

class IPv4PublicNetworkPools(_message.Message):
    __slots__ = ["pools"]
    POOLS_FIELD_NUMBER: _ClassVar[int]
    pools: _containers.RepeatedCompositeFieldContainer[NetworkPool]
    def __init__(self, pools: _Optional[_Iterable[_Union[NetworkPool, _Mapping]]] = ...) -> None: ...

class NetworkPool(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class NetworkStatus(_message.Message):
    __slots__ = ["state", "default_route_table_id"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[NetworkStatus.State]
        CREATING: _ClassVar[NetworkStatus.State]
        READY: _ClassVar[NetworkStatus.State]
        DELETING: _ClassVar[NetworkStatus.State]
    STATE_UNSPECIFIED: NetworkStatus.State
    CREATING: NetworkStatus.State
    READY: NetworkStatus.State
    DELETING: NetworkStatus.State
    STATE_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_ROUTE_TABLE_ID_FIELD_NUMBER: _ClassVar[int]
    state: NetworkStatus.State
    default_route_table_id: str
    def __init__(self, state: _Optional[_Union[NetworkStatus.State, str]] = ..., default_route_table_id: _Optional[str] = ...) -> None: ...



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class TargetGroup(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: TargetGroupSpec
    status: TargetGroupStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[TargetGroupSpec, _Mapping]] = ..., status: _Optional[_Union[TargetGroupStatus, _Mapping]] = ...) -> None: ...

class TargetGroupSpec(_message.Message):
    __slots__ = ["targets"]
    TARGETS_FIELD_NUMBER: _ClassVar[int]
    targets: _containers.RepeatedCompositeFieldContainer[Target]
    def __init__(self, targets: _Optional[_Iterable[_Union[Target, _Mapping]]] = ...) -> None: ...

class Target(_message.Message):
    __slots__ = ["compute_instance"]
    COMPUTE_INSTANCE_FIELD_NUMBER: _ClassVar[int]
    compute_instance: ComputeInstance
    def __init__(self, compute_instance: _Optional[_Union[ComputeInstance, _Mapping]] = ...) -> None: ...

class ComputeInstance(_message.Message):
    __slots__ = ["id", "network_interface_name"]
    ID_FIELD_NUMBER: _ClassVar[int]
    NETWORK_INTERFACE_NAME_FIELD_NUMBER: _ClassVar[int]
    id: str
    network_interface_name: str
    def __init__(self, id: _Optional[str] = ..., network_interface_name: _Optional[str] = ...) -> None: ...

class TargetGroupStatus(_message.Message):
    __slots__ = ["load_balancer_ids", "target_statuses"]
    LOAD_BALANCER_IDS_FIELD_NUMBER: _ClassVar[int]
    TARGET_STATUSES_FIELD_NUMBER: _ClassVar[int]
    load_balancer_ids: _containers.RepeatedScalarFieldContainer[str]
    target_statuses: _containers.RepeatedCompositeFieldContainer[TargetStatus]
    def __init__(self, load_balancer_ids: _Optional[_Iterable[str]] = ..., target_statuses: _Optional[_Iterable[_Union[TargetStatus, _Mapping]]] = ...) -> None: ...

class TargetStatus(_message.Message):
    __slots__ = ["compute_instance_id", "target_state"]
    class TargetState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        TARGET_STATE_UNSPECIFIED: _ClassVar[TargetStatus.TargetState]
        READY: _ClassVar[TargetStatus.TargetState]
        DISABLED: _ClassVar[TargetStatus.TargetState]
        DELETED: _ClassVar[TargetStatus.TargetState]
    TARGET_STATE_UNSPECIFIED: TargetStatus.TargetState
    READY: TargetStatus.TargetState
    DISABLED: TargetStatus.TargetState
    DELETED: TargetStatus.TargetState
    COMPUTE_INSTANCE_ID_FIELD_NUMBER: _ClassVar[int]
    TARGET_STATE_FIELD_NUMBER: _ClassVar[int]
    compute_instance_id: str
    target_state: TargetStatus.TargetState
    def __init__(self, compute_instance_id: _Optional[str] = ..., target_state: _Optional[_Union[TargetStatus.TargetState, str]] = ...) -> None: ...



# 
# Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
# 

"""Auto-generated Nebius SDK package for ``nebius.audit.v2``"""

import builtins as builtins
import collections.abc as abc
import datetime as datetime
import google.protobuf.descriptor as descriptor_1
import google.protobuf.message as message_1
import google.protobuf.struct_pb2 as struct_pb2
import google.protobuf.timestamp_pb2 as timestamp_pb2
import google.rpc.code_pb2 as code_pb2
import nebius.aio.client as client
import nebius.aio.request as request_1
import nebius.aio.request_kwargs as request_kwargs
import nebius.api.nebius.audit.v2.access_token_pb2 as access_token_pb2
import nebius.api.nebius.audit.v2.audit_event_pb2 as audit_event_pb2
import nebius.api.nebius.audit.v2.audit_event_service_pb2 as audit_event_service_pb2
import nebius.api.nebius.audit.v2.authentication_pb2 as authentication_pb2
import nebius.api.nebius.audit.v2.authentication_type_pb2 as authentication_type_pb2
import nebius.api.nebius.audit.v2.authorization_pb2 as authorization_pb2
import nebius.api.nebius.audit.v2.federation_pb2 as federation_pb2
import nebius.api.nebius.audit.v2.region_pb2 as region_pb2
import nebius.api.nebius.audit.v2.request_pb2 as request_pb2
import nebius.api.nebius.audit.v2.resource_metadata_pb2 as resource_metadata_pb2
import nebius.api.nebius.audit.v2.resource_pb2 as resource_pb2
import nebius.api.nebius.audit.v2.resource_state_pb2 as resource_state_pb2
import nebius.api.nebius.audit.v2.response_pb2 as response_pb2
import nebius.api.nebius.audit.v2.service_pb2 as service_pb2
import nebius.api.nebius.audit.v2.static_key_pb2 as static_key_pb2
import nebius.api.nebius.audit.v2.status_pb2 as status_pb2
import nebius.api.nebius.audit.v2.subject_pb2 as subject_pb2
import nebius.base.protos.descriptor as descriptor
import nebius.base.protos.pb_classes as pb_classes
import nebius.base.protos.pb_enum as pb_enum
import nebius.base.protos.unset as unset
import nebius.base.protos.well_known as well_known_1
import typing as typing
import typing_extensions as typing_extensions
#@ local imports here @#

# file: nebius/audit/v2/access_token.proto
class AccessToken(pb_classes.Message):
    __PB2_CLASS__ = access_token_pb2.AccessToken
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.AccessToken",access_token_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        masked_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(masked_token, unset.UnsetType):
            self.masked_token = masked_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "masked_token",
        ]
    
    @builtins.property
    def masked_token(self) -> "builtins.str":
        """
        Token without signature.
        """
        
        return super()._get_field("masked_token", explicit_presence=False,
        )
    @masked_token.setter
    def masked_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("masked_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "masked_token":"masked_token",
    }
    
# file: nebius/audit/v2/authentication_type.proto
class AuthenticationType(pb_enum.Enum):
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.audit.v2.AuthenticationType",authentication_type_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
    AUTHENTICATION_TYPE_UNSPECIFIED = 0
    ACCESS_TOKEN = 1
    STATIC_KEY = 2

# file: nebius/audit/v2/federation.proto
class Federation(pb_classes.Message):
    __PB2_CLASS__ = federation_pb2.Federation
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.Federation",federation_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "name",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        Unique identifier of the federation.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def name(self) -> "builtins.str":
        """
        Human-readable name of the federation.
        """
        
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "name":"name",
    }
    
# file: nebius/audit/v2/static_key.proto
class StaticKey(pb_classes.Message):
    __PB2_CLASS__ = static_key_pb2.StaticKey
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.StaticKey",static_key_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        Unique identifier of static key.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    
# file: nebius/audit/v2/subject.proto
class Subject(pb_classes.Message):
    __PB2_CLASS__ = subject_pb2.Subject
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.Subject",subject_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class __OneOfClass_id__(pb_classes.OneOf):
        name: builtins.str= "id"
        
        def __init__(self, msg: "Subject") -> None:
            super().__init__()
            self._message: "Subject" = msg
    
    class __OneOfClass_id_service_account_id__(__OneOfClass_id__):
        field: typing.Literal["service_account_id"] = "service_account_id"
        
        def __init__(self, msg: "Subject") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "builtins.str":
            return self._message.service_account_id
    
    class __OneOfClass_id_tenant_user_id__(__OneOfClass_id__):
        field: typing.Literal["tenant_user_id"] = "tenant_user_id"
        
        def __init__(self, msg: "Subject") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "builtins.str":
            return self._message.tenant_user_id
    
    @builtins.property
    def id(self) -> __OneOfClass_id_service_account_id__|__OneOfClass_id_tenant_user_id__|None:
        field_name_1: str|None = super().which_field_in_oneof("id")
        match field_name_1:
            case "service_account_id":
                return self.__OneOfClass_id_service_account_id__(self)
            case "tenant_user_id":
                return self.__OneOfClass_id_tenant_user_id__(self)
            case None:
                return None
            case _:
                raise pb_classes.OneOfMatchError(field_name_1)
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        service_account_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        tenant_user_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(service_account_id, unset.UnsetType):
            self.service_account_id = service_account_id
        if not isinstance(tenant_user_id, unset.UnsetType):
            self.tenant_user_id = tenant_user_id
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "service_account_id",
            "tenant_user_id",
            "name",
            "id",
        ]
    
    @builtins.property
    def service_account_id(self) -> "builtins.str|None":
        """
        This field is filled in if the call was made on behalf of a service account.
        """
        
        return super()._get_field("service_account_id", explicit_presence=True,
        )
    @service_account_id.setter
    def service_account_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("service_account_id",value,explicit_presence=True,
        )
    
    @builtins.property
    def tenant_user_id(self) -> "builtins.str|None":
        """
        This field is filled in if the call was made on behalf of a user.
        """
        
        return super()._get_field("tenant_user_id", explicit_presence=True,
        )
    @tenant_user_id.setter
    def tenant_user_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("tenant_user_id",value,explicit_presence=True,
        )
    
    @builtins.property
    def name(self) -> "builtins.str":
        """
        Username of the subject.
        """
        
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "service_account_id":"service_account_id",
        "tenant_user_id":"tenant_user_id",
        "name":"name",
        "id":"id",
    }
    
# file: nebius/audit/v2/authentication.proto
class Authentication(pb_classes.Message):
    __PB2_CLASS__ = authentication_pb2.Authentication
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.Authentication",authentication_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class __OneOfClass_credential__(pb_classes.OneOf):
        name: builtins.str= "credential"
        
        def __init__(self, msg: "Authentication") -> None:
            super().__init__()
            self._message: "Authentication" = msg
    
    class __OneOfClass_credential_token_credential__(__OneOfClass_credential__):
        field: typing.Literal["token_credential"] = "token_credential"
        
        def __init__(self, msg: "Authentication") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "AccessToken":
            return self._message.token_credential
    
    class __OneOfClass_credential_static_key_credential__(__OneOfClass_credential__):
        field: typing.Literal["static_key_credential"] = "static_key_credential"
        
        def __init__(self, msg: "Authentication") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "StaticKey":
            return self._message.static_key_credential
    
    @builtins.property
    def credential(self) -> __OneOfClass_credential_token_credential__|__OneOfClass_credential_static_key_credential__|None:
        field_name_1: str|None = super().which_field_in_oneof("credential")
        match field_name_1:
            case "token_credential":
                return self.__OneOfClass_credential_token_credential__(self)
            case "static_key_credential":
                return self.__OneOfClass_credential_static_key_credential__(self)
            case None:
                return None
            case _:
                raise pb_classes.OneOfMatchError(field_name_1)
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        authenticated: "builtins.bool|None|unset.UnsetType" = unset.Unset,
        subject: "Subject|subject_pb2.Subject|None|unset.UnsetType" = unset.Unset,
        federation: "Federation|federation_pb2.Federation|None|unset.UnsetType" = unset.Unset,
        authentication_type: "AuthenticationType|authentication_type_pb2.AuthenticationType|None|unset.UnsetType" = unset.Unset,
        token_credential: "AccessToken|access_token_pb2.AccessToken|None|unset.UnsetType" = unset.Unset,
        static_key_credential: "StaticKey|static_key_pb2.StaticKey|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(authenticated, unset.UnsetType):
            self.authenticated = authenticated
        if not isinstance(subject, unset.UnsetType):
            self.subject = subject
        if not isinstance(federation, unset.UnsetType):
            self.federation = federation
        if not isinstance(authentication_type, unset.UnsetType):
            self.authentication_type = authentication_type
        if not isinstance(token_credential, unset.UnsetType):
            self.token_credential = token_credential
        if not isinstance(static_key_credential, unset.UnsetType):
            self.static_key_credential = static_key_credential
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "authenticated",
            "subject",
            "federation",
            "authentication_type",
            "token_credential",
            "static_key_credential",
            "credential",
        ]
    
    @builtins.property
    def authenticated(self) -> "builtins.bool":
        """
        Answers the question whether or not the subject has been successfully authenticated.
        """
        
        return super()._get_field("authenticated", explicit_presence=False,
        )
    @authenticated.setter
    def authenticated(self, value: "builtins.bool|None") -> None:
        return super()._set_field("authenticated",value,explicit_presence=False,
        )
    
    @builtins.property
    def subject(self) -> "Subject":
        """
        Contains information about the subject of the call.
        """
        
        return super()._get_field("subject", explicit_presence=False,
        wrap=Subject,
        )
    @subject.setter
    def subject(self, value: "Subject|subject_pb2.Subject|None") -> None:
        return super()._set_field("subject",value,explicit_presence=False,
        )
    
    @builtins.property
    def federation(self) -> "Federation":
        """
        Information on the federation within which the subject exists.
        """
        
        return super()._get_field("federation", explicit_presence=False,
        wrap=Federation,
        )
    @federation.setter
    def federation(self, value: "Federation|federation_pb2.Federation|None") -> None:
        return super()._set_field("federation",value,explicit_presence=False,
        )
    
    @builtins.property
    def authentication_type(self) -> "AuthenticationType":
        """
        Describes the type of authentication.
        """
        
        return super()._get_field("authentication_type", explicit_presence=False,
        wrap=AuthenticationType,
        )
    @authentication_type.setter
    def authentication_type(self, value: "AuthenticationType|authentication_type_pb2.AuthenticationType|None") -> None:
        return super()._set_field("authentication_type",value,explicit_presence=False,
        )
    
    @builtins.property
    def token_credential(self) -> "AccessToken|None":
        """
        Information about token.
        """
        
        return super()._get_field("token_credential", explicit_presence=True,
        wrap=AccessToken,
        )
    @token_credential.setter
    def token_credential(self, value: "AccessToken|access_token_pb2.AccessToken|None") -> None:
        return super()._set_field("token_credential",value,explicit_presence=True,
        )
    
    @builtins.property
    def static_key_credential(self) -> "StaticKey|None":
        """
        Information about static key.
        """
        
        return super()._get_field("static_key_credential", explicit_presence=True,
        wrap=StaticKey,
        )
    @static_key_credential.setter
    def static_key_credential(self, value: "StaticKey|static_key_pb2.StaticKey|None") -> None:
        return super()._set_field("static_key_credential",value,explicit_presence=True,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "authenticated":"authenticated",
        "subject":"subject",
        "federation":"federation",
        "authentication_type":"authentication_type",
        "token_credential":"token_credential",
        "static_key_credential":"static_key_credential",
        "credential":"credential",
    }
    
# file: nebius/audit/v2/authorization.proto
class Authorization(pb_classes.Message):
    __PB2_CLASS__ = authorization_pb2.Authorization
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.Authorization",authorization_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        authorized: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(authorized, unset.UnsetType):
            self.authorized = authorized
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "authorized",
        ]
    
    @builtins.property
    def authorized(self) -> "builtins.bool":
        """
        Field is required. We do not use «(buf.validate.field).required = true» here as false value considered as unset (validation violation)
        """
        
        return super()._get_field("authorized", explicit_presence=False,
        )
    @authorized.setter
    def authorized(self, value: "builtins.bool|None") -> None:
        return super()._set_field("authorized",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "authorized":"authorized",
    }
    
# file: nebius/audit/v2/region.proto
class Region(pb_classes.Message):
    __PB2_CLASS__ = region_pb2.Region
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.Region",region_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        """
        Region name
        """
        
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
# file: nebius/audit/v2/request.proto
class Request(pb_classes.Message):
    __PB2_CLASS__ = request_pb2.Request
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.Request",request_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        client_ip: "builtins.str|None|unset.UnsetType" = unset.Unset,
        user_agent: "builtins.str|None|unset.UnsetType" = unset.Unset,
        request_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        parameters: "struct_pb2.Struct|None|unset.UnsetType" = unset.Unset,
        idempotency_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        trace_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        ja3_fingerprint: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(client_ip, unset.UnsetType):
            self.client_ip = client_ip
        if not isinstance(user_agent, unset.UnsetType):
            self.user_agent = user_agent
        if not isinstance(request_id, unset.UnsetType):
            self.request_id = request_id
        if not isinstance(parameters, unset.UnsetType):
            self.parameters = parameters
        if not isinstance(idempotency_id, unset.UnsetType):
            self.idempotency_id = idempotency_id
        if not isinstance(trace_id, unset.UnsetType):
            self.trace_id = trace_id
        if not isinstance(ja3_fingerprint, unset.UnsetType):
            self.ja3_fingerprint = ja3_fingerprint
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "client_ip",
            "user_agent",
            "request_id",
            "parameters",
            "idempotency_id",
            "trace_id",
            "ja3_fingerprint",
        ]
    
    @builtins.property
    def client_ip(self) -> "builtins.str":
        """
        Ip of the subject who initiated the RPC call.
        """
        
        return super()._get_field("client_ip", explicit_presence=False,
        )
    @client_ip.setter
    def client_ip(self, value: "builtins.str|None") -> None:
        return super()._set_field("client_ip",value,explicit_presence=False,
        )
    
    @builtins.property
    def user_agent(self) -> "builtins.str":
        """
        User agent of the subject who initiated operation.
        """
        
        return super()._get_field("user_agent", explicit_presence=False,
        )
    @user_agent.setter
    def user_agent(self, value: "builtins.str|None") -> None:
        return super()._set_field("user_agent",value,explicit_presence=False,
        )
    
    @builtins.property
    def request_id(self) -> "builtins.str":
        """
        Unique identifier of the request.
        """
        
        return super()._get_field("request_id", explicit_presence=False,
        )
    @request_id.setter
    def request_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("request_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def parameters(self) -> "struct_pb2.Struct|None":
        """
        Parameters of the request.
        """
        
        return super()._get_field("parameters", explicit_presence=True,
        )
    @parameters.setter
    def parameters(self, value: "struct_pb2.Struct|None") -> None:
        return super()._set_field("parameters",value,explicit_presence=True,
        )
    
    @builtins.property
    def idempotency_id(self) -> "builtins.str":
        """
        Unique key that the server uses to recognize subsequent retries of the same request.
        """
        
        return super()._get_field("idempotency_id", explicit_presence=False,
        )
    @idempotency_id.setter
    def idempotency_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("idempotency_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def trace_id(self) -> "builtins.str":
        """
        A unique identifier that allows you to link calls to different services within an operation.
        """
        
        return super()._get_field("trace_id", explicit_presence=False,
        )
    @trace_id.setter
    def trace_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("trace_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def ja3_fingerprint(self) -> "builtins.str":
        """
        Client Ja3 fingerprint
        """
        
        return super()._get_field("ja3_fingerprint", explicit_presence=False,
        )
    @ja3_fingerprint.setter
    def ja3_fingerprint(self, value: "builtins.str|None") -> None:
        return super()._set_field("ja3_fingerprint",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "client_ip":"client_ip",
        "user_agent":"user_agent",
        "request_id":"request_id",
        "parameters":"parameters",
        "idempotency_id":"idempotency_id",
        "trace_id":"trace_id",
        "ja3_fingerprint":"ja3_fingerprint",
    }
    
# file: nebius/audit/v2/resource_metadata.proto
class ResourceMetadata(pb_classes.Message):
    __PB2_CLASS__ = resource_metadata_pb2.ResourceMetadata
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.ResourceMetadata",resource_metadata_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
        type: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(name, unset.UnsetType):
            self.name = name
        if not isinstance(type, unset.UnsetType):
            self.type = type
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "name",
            "type",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        Unique identifier of the resource.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def name(self) -> "builtins.str":
        """
        Name of the resource.
        """
        
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    @builtins.property
    def type(self) -> "builtins.str":
        """
        Type of the resource.
        """
        
        return super()._get_field("type", explicit_presence=False,
        )
    @type.setter
    def type(self, value: "builtins.str|None") -> None:
        return super()._set_field("type",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "name":"name",
        "type":"type",
    }
    
# file: nebius/audit/v2/resource_state.proto
class ResourceState(pb_classes.Message):
    __PB2_CLASS__ = resource_state_pb2.ResourceState
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.ResourceState",resource_state_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        previous: "struct_pb2.Struct|None|unset.UnsetType" = unset.Unset,
        current: "struct_pb2.Struct|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(previous, unset.UnsetType):
            self.previous = previous
        if not isinstance(current, unset.UnsetType):
            self.current = current
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "previous",
            "current",
        ]
    
    @builtins.property
    def previous(self) -> "struct_pb2.Struct|None":
        """
        The state of the resource before the change in the call.
        """
        
        return super()._get_field("previous", explicit_presence=True,
        )
    @previous.setter
    def previous(self, value: "struct_pb2.Struct|None") -> None:
        return super()._set_field("previous",value,explicit_presence=True,
        )
    
    @builtins.property
    def current(self) -> "struct_pb2.Struct|None":
        """
        The state of the resource after the change in the call.
        """
        
        return super()._get_field("current", explicit_presence=True,
        )
    @current.setter
    def current(self, value: "struct_pb2.Struct|None") -> None:
        return super()._set_field("current",value,explicit_presence=True,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "previous":"previous",
        "current":"current",
    }
    
# file: nebius/audit/v2/resource.proto
class Resource(pb_classes.Message):
    __PB2_CLASS__ = resource_pb2.Resource
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.Resource",resource_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "ResourceMetadata|resource_metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        state: "ResourceState|resource_state_pb2.ResourceState|None|unset.UnsetType" = unset.Unset,
        hierarchy: "abc.Iterable[ResourceMetadata]|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(state, unset.UnsetType):
            self.state = state
        if not isinstance(hierarchy, unset.UnsetType):
            self.hierarchy = hierarchy
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "state",
            "hierarchy",
        ]
    
    @builtins.property
    def metadata(self) -> "ResourceMetadata":
        """
        Provides basic information about the resource.
        """
        
        return super()._get_field("metadata", explicit_presence=False,
        wrap=ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "ResourceMetadata|resource_metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def state(self) -> "ResourceState":
        """
        Gives an understanding of how the resource was changed in the call.
        """
        
        return super()._get_field("state", explicit_presence=False,
        wrap=ResourceState,
        )
    @state.setter
    def state(self, value: "ResourceState|resource_state_pb2.ResourceState|None") -> None:
        return super()._set_field("state",value,explicit_presence=False,
        )
    
    @builtins.property
    def hierarchy(self) -> "abc.MutableSequence[ResourceMetadata]":
        """
        Top-down resources hierarchy from tenant to resource.
        """
        
        return super()._get_field("hierarchy", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(ResourceMetadata,None,None),
        )
    @hierarchy.setter
    def hierarchy(self, value: "abc.Iterable[ResourceMetadata]|None") -> None:
        return super()._set_field("hierarchy",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "state":"state",
        "hierarchy":"hierarchy",
    }
    
# file: nebius/audit/v2/response.proto
class Response(pb_classes.Message):
    __PB2_CLASS__ = response_pb2.Response
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.Response",response_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        status_code: "code_pb2.Code|None|unset.UnsetType" = unset.Unset,
        error_message: "builtins.str|None|unset.UnsetType" = unset.Unset,
        payload: "struct_pb2.Struct|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(status_code, unset.UnsetType):
            self.status_code = status_code
        if not isinstance(error_message, unset.UnsetType):
            self.error_message = error_message
        if not isinstance(payload, unset.UnsetType):
            self.payload = payload
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "status_code",
            "error_message",
            "payload",
        ]
    
    @builtins.property
    def status_code(self) -> "code_pb2.Code":
        """
        The status that the server returned to the client.
        """
        
        return super()._get_field("status_code", explicit_presence=False,
        )
    @status_code.setter
    def status_code(self, value: "code_pb2.Code|None") -> None:
        return super()._set_field("status_code",value,explicit_presence=False,
        )
    
    @builtins.property
    def error_message(self) -> "builtins.str":
        """
        Error message if the call was not successful.
        """
        
        return super()._get_field("error_message", explicit_presence=False,
        )
    @error_message.setter
    def error_message(self, value: "builtins.str|None") -> None:
        return super()._set_field("error_message",value,explicit_presence=False,
        )
    
    @builtins.property
    def payload(self) -> "struct_pb2.Struct|None":
        """
        The payload of the response.
        """
        
        return super()._get_field("payload", explicit_presence=True,
        )
    @payload.setter
    def payload(self, value: "struct_pb2.Struct|None") -> None:
        return super()._set_field("payload",value,explicit_presence=True,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "status_code":"status_code",
        "error_message":"error_message",
        "payload":"payload",
    }
    
# file: nebius/audit/v2/service.proto
class Service(pb_classes.Message):
    __PB2_CLASS__ = service_pb2.Service
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.Service",service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
# file: nebius/audit/v2/status.proto
class Status(pb_enum.Enum):
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.audit.v2.Status",status_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
    RESPONSE_STATUS_UNSPECIFIED = 0
    STARTED = 1
    """
    Asynchronous action started
    """
    
    DONE = 2
    """
    Action finished successfully
    """
    
    ERROR = 3
    """
    Action finished with error
    """
    

# file: nebius/audit/v2/audit_event.proto
class AuditEvent(pb_classes.Message):
    __PB2_CLASS__ = audit_event_pb2.AuditEvent
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.AuditEvent",audit_event_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
        "time": well_known_1.ts_mask,
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        source: "builtins.str|None|unset.UnsetType" = unset.Unset,
        spec_version: "builtins.str|None|unset.UnsetType" = unset.Unset,
        type: "builtins.str|None|unset.UnsetType" = unset.Unset,
        service: "Service|service_pb2.Service|None|unset.UnsetType" = unset.Unset,
        action: "builtins.str|None|unset.UnsetType" = unset.Unset,
        time: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
        event_version: "builtins.str|None|unset.UnsetType" = unset.Unset,
        authentication: "Authentication|authentication_pb2.Authentication|None|unset.UnsetType" = unset.Unset,
        authorization: "Authorization|authorization_pb2.Authorization|None|unset.UnsetType" = unset.Unset,
        resource: "Resource|resource_pb2.Resource|None|unset.UnsetType" = unset.Unset,
        request: "Request|request_pb2.Request|None|unset.UnsetType" = unset.Unset,
        response: "Response|response_pb2.Response|None|unset.UnsetType" = unset.Unset,
        status: "Status|status_pb2.Status|None|unset.UnsetType" = unset.Unset,
        project_region: "Region|region_pb2.Region|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(source, unset.UnsetType):
            self.source = source
        if not isinstance(spec_version, unset.UnsetType):
            self.spec_version = spec_version
        if not isinstance(type, unset.UnsetType):
            self.type = type
        if not isinstance(service, unset.UnsetType):
            self.service = service
        if not isinstance(action, unset.UnsetType):
            self.action = action
        if not isinstance(time, unset.UnsetType):
            self.time = time
        if not isinstance(event_version, unset.UnsetType):
            self.event_version = event_version
        if not isinstance(authentication, unset.UnsetType):
            self.authentication = authentication
        if not isinstance(authorization, unset.UnsetType):
            self.authorization = authorization
        if not isinstance(resource, unset.UnsetType):
            self.resource = resource
        if not isinstance(request, unset.UnsetType):
            self.request = request
        if not isinstance(response, unset.UnsetType):
            self.response = response
        if not isinstance(status, unset.UnsetType):
            self.status = status
        if not isinstance(project_region, unset.UnsetType):
            self.project_region = project_region
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "source",
            "spec_version",
            "type",
            "service",
            "action",
            "time",
            "event_version",
            "authentication",
            "authorization",
            "resource",
            "request",
            "response",
            "status",
            "project_region",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        Unique event id.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def source(self) -> "builtins.str":
        """
        Identifies the context in which an event happened. Presented in the form of URI.
        """
        
        return super()._get_field("source", explicit_presence=False,
        )
    @source.setter
    def source(self, value: "builtins.str|None") -> None:
        return super()._set_field("source",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec_version(self) -> "builtins.str":
        """
        Version of CloudEvents spec. See https://github.com/cloudevents/spec/blob/main/cloudevents/spec.md#specversion
        """
        
        return super()._get_field("spec_version", explicit_presence=False,
        )
    @spec_version.setter
    def spec_version(self, value: "builtins.str|None") -> None:
        return super()._set_field("spec_version",value,explicit_presence=False,
        )
    
    @builtins.property
    def type(self) -> "builtins.str":
        """
        .. role:: raw-html-m2r(raw)
           :format: html
        
        
        The type of event related to the originating occurrence. Formed according to template: "ai.nebius.\\ :raw-html-m2r:`<serviceName>`.\\ :raw-html-m2r:`<resourceType>`.\\ :raw-html-m2r:`<action>`\\ "
        """
        
        return super()._get_field("type", explicit_presence=False,
        )
    @type.setter
    def type(self, value: "builtins.str|None") -> None:
        return super()._set_field("type",value,explicit_presence=False,
        )
    
    @builtins.property
    def service(self) -> "Service":
        """
        Indicates the service that generated the event.
        """
        
        return super()._get_field("service", explicit_presence=False,
        wrap=Service,
        )
    @service.setter
    def service(self, value: "Service|service_pb2.Service|None") -> None:
        return super()._set_field("service",value,explicit_presence=False,
        )
    
    @builtins.property
    def action(self) -> "builtins.str":
        """
        Indicates an action taken on the resource.
        """
        
        return super()._get_field("action", explicit_presence=False,
        )
    @action.setter
    def action(self, value: "builtins.str|None") -> None:
        return super()._set_field("action",value,explicit_presence=False,
        )
    
    @builtins.property
    def time(self) -> "datetime.datetime":
        """
        Timestamp of when the occurrence happened.
        """
        
        return super()._get_field("time", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @time.setter
    def time(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("time",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    @builtins.property
    def event_version(self) -> "builtins.str":
        """
        Audit event version, in "X.Y" format, events within different Y's are backwards compatible, events with different X's are not.
        """
        
        return super()._get_field("event_version", explicit_presence=False,
        )
    @event_version.setter
    def event_version(self, value: "builtins.str|None") -> None:
        return super()._set_field("event_version",value,explicit_presence=False,
        )
    
    @builtins.property
    def authentication(self) -> "Authentication":
        """
        Represents the subject of an API call.
        """
        
        return super()._get_field("authentication", explicit_presence=False,
        wrap=Authentication,
        )
    @authentication.setter
    def authentication(self, value: "Authentication|authentication_pb2.Authentication|None") -> None:
        return super()._set_field("authentication",value,explicit_presence=False,
        )
    
    @builtins.property
    def authorization(self) -> "Authorization":
        """
        Represents permissions that have been verified for the subject as part of the API call.
        """
        
        return super()._get_field("authorization", explicit_presence=False,
        wrap=Authorization,
        )
    @authorization.setter
    def authorization(self, value: "Authorization|authorization_pb2.Authorization|None") -> None:
        return super()._set_field("authorization",value,explicit_presence=False,
        )
    
    @builtins.property
    def resource(self) -> "Resource":
        """
        A resource that was read or modified as part of the operation. Can be empty if no resource accessed or changed.
        """
        
        return super()._get_field("resource", explicit_presence=False,
        wrap=Resource,
        )
    @resource.setter
    def resource(self, value: "Resource|resource_pb2.Resource|None") -> None:
        return super()._set_field("resource",value,explicit_presence=False,
        )
    
    @builtins.property
    def request(self) -> "Request":
        """
        Contains information about the request from the customer.
        """
        
        return super()._get_field("request", explicit_presence=False,
        wrap=Request,
        )
    @request.setter
    def request(self, value: "Request|request_pb2.Request|None") -> None:
        return super()._set_field("request",value,explicit_presence=False,
        )
    
    @builtins.property
    def response(self) -> "Response":
        """
        Contains information about the response from the server.
        """
        
        return super()._get_field("response", explicit_presence=False,
        wrap=Response,
        )
    @response.setter
    def response(self, value: "Response|response_pb2.Response|None") -> None:
        return super()._set_field("response",value,explicit_presence=False,
        )
    
    @builtins.property
    def status(self) -> "Status":
        """
        The async operation status
        """
        
        return super()._get_field("status", explicit_presence=False,
        wrap=Status,
        )
    @status.setter
    def status(self, value: "Status|status_pb2.Status|None") -> None:
        return super()._set_field("status",value,explicit_presence=False,
        )
    
    @builtins.property
    def project_region(self) -> "Region":
        """
        Region of an application
        """
        
        return super()._get_field("project_region", explicit_presence=False,
        wrap=Region,
        )
    @project_region.setter
    def project_region(self, value: "Region|region_pb2.Region|None") -> None:
        return super()._set_field("project_region",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "source":"source",
        "spec_version":"spec_version",
        "type":"type",
        "service":"service",
        "action":"action",
        "time":"time",
        "event_version":"event_version",
        "authentication":"authentication",
        "authorization":"authorization",
        "resource":"resource",
        "request":"request",
        "response":"response",
        "status":"status",
        "project_region":"project_region",
    }
    
# file: nebius/audit/v2/audit_event_service.proto
class ListAuditEventRequest(pb_classes.Message):
    __PB2_CLASS__ = audit_event_service_pb2.ListAuditEventRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.ListAuditEventRequest",audit_event_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
        "start": well_known_1.ts_mask,
        "end": well_known_1.ts_mask,
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        page_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
        start: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
        end: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
        page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
        filter: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
        if not isinstance(page_size, unset.UnsetType):
            self.page_size = page_size
        if not isinstance(start, unset.UnsetType):
            self.start = start
        if not isinstance(end, unset.UnsetType):
            self.end = end
        if not isinstance(page_token, unset.UnsetType):
            self.page_token = page_token
        if not isinstance(filter, unset.UnsetType):
            self.filter = filter
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
            "page_size",
            "start",
            "end",
            "page_token",
            "filter",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        A tenant id must be provided
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_size(self) -> "builtins.int":
        return super()._get_field("page_size", explicit_presence=False,
        )
    @page_size.setter
    def page_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("page_size",value,explicit_presence=False,
        )
    
    @builtins.property
    def start(self) -> "datetime.datetime":
        return super()._get_field("start", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @start.setter
    def start(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("start",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    @builtins.property
    def end(self) -> "datetime.datetime":
        return super()._get_field("end", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @end.setter
    def end(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("end",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    @builtins.property
    def page_token(self) -> "builtins.str":
        return super()._get_field("page_token", explicit_presence=False,
        )
    @page_token.setter
    def page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("page_token",value,explicit_presence=False,
        )
    
    @builtins.property
    def filter(self) -> "builtins.str":
        """
        Example:
        service.name = 'iam' AND resource.hierarchy.id:'container-e0t' AND regex(resource.metadata.name, '^.\\ *test.*\\ $')
        
        Supported filters:
        "=" - equals
        "!=" - not equals
        ":" - contains
        regex - regular expression
        
        Fields that can be used for filtering:
        action
        authentication.static_key_credential.id
        authentication.subject.name
        authentication.subject.service_account_id
        authentication.subject.tenant_user_id
        authentication.token_credential.masked_token
        project_region.name
        resource.hierarchy.id
        resource.hierarchy.name
        resource.metadata.id
        resource.metadata.name
        resource.metadata.type
        service.name
        type
        status
        """
        
        return super()._get_field("filter", explicit_presence=False,
        )
    @filter.setter
    def filter(self, value: "builtins.str|None") -> None:
        return super()._set_field("filter",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
        "page_size":"page_size",
        "start":"start",
        "end":"end",
        "page_token":"page_token",
        "filter":"filter",
    }
    
class ListAuditEventResponse(pb_classes.Message):
    __PB2_CLASS__ = audit_event_service_pb2.ListAuditEventResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.audit.v2.ListAuditEventResponse",audit_event_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        items: "abc.Iterable[AuditEvent]|None|unset.UnsetType" = unset.Unset,
        next_page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(items, unset.UnsetType):
            self.items = items
        if not isinstance(next_page_token, unset.UnsetType):
            self.next_page_token = next_page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "items",
            "next_page_token",
        ]
    
    @builtins.property
    def items(self) -> "abc.MutableSequence[AuditEvent]":
        return super()._get_field("items", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(AuditEvent,None,None),
        )
    @items.setter
    def items(self, value: "abc.Iterable[AuditEvent]|None") -> None:
        return super()._set_field("items",value,explicit_presence=False,
        )
    
    @builtins.property
    def next_page_token(self) -> "builtins.str":
        return super()._get_field("next_page_token", explicit_presence=False,
        )
    @next_page_token.setter
    def next_page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("next_page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "items":"items",
        "next_page_token":"next_page_token",
    }
    

class AuditEventServiceClient(client.Client):
    """
    This class provides the client methods for the ``.nebius.audit.v2.AuditEventService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.audit.v2.AuditEventService",audit_event_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.audit.v2.AuditEventService"
    
    def list(self,
        request: "ListAuditEventRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["ListAuditEventRequest","ListAuditEventResponse"]:
        """
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.audit.v2.ListAuditEventRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.audit.v2.ListAuditEventResponse`.
        """
        
        return super().request(
            method="List",
            request=request,
            result_pb2_class=audit_event_service_pb2.ListAuditEventResponse,
            result_wrapper=pb_classes.simple_wrapper(ListAuditEventResponse),
            **kwargs,
        )
    

__all__ = [
    #@ local import names here @#
    "AccessToken",
    "AuthenticationType",
    "Federation",
    "StaticKey",
    "Subject",
    "Authentication",
    "Authorization",
    "Region",
    "Request",
    "ResourceMetadata",
    "ResourceState",
    "Resource",
    "Response",
    "Service",
    "Status",
    "AuditEvent",
    "ListAuditEventRequest",
    "ListAuditEventResponse",
    "AuditEventServiceClient",
]



"""Helpers for working with long-running operations.

This module provides an :class:`Operation` wrapper that normalizes different
versions of the service operation protobuf and exposes convenient helpers for
polling, synchronous waiting, and inspecting operation metadata.

The wrapper accepts operation protobufs from either the current v1 API or an
older v1alpha1 variant and routes calls to the corresponding operation
service client.
"""

from asyncio import sleep
from datetime import datetime, timedelta
from time import time
from typing import Generic, TypeVar

from grpc import StatusCode
from typing_extensions import Unpack

from nebius.aio.abc import ClientChannelInterface
from nebius.aio.request import DEFAULT_TIMEOUT
from nebius.aio.request_kwargs import RequestKwargs, RequestKwargsForOperation
from nebius.base.error import SDKError
from nebius.base.protos.unset import Unset, UnsetType
from nebius.base.protos.well_known import local_timezone

from .constant_channel import Constant
from .request_status import RequestStatus

OperationPb = TypeVar("OperationPb")
"""
A convenience wrapper around operation protobufs.
Either :class:`nebius.api.nebius.common.v1.Operation` or
:class:`nebius.api.nebius.common.v1alpha1.Operation`, or their protobuf classes.
"""
T = TypeVar("T")


class Operation(Generic[OperationPb]):
    """A convenience wrapper around operation protobufs.

    The :class:`Operation` wrapper normalizes
    :class:`nebius.api.nebius.common.v1.Operation`
    and :class:`nebius.api.nebius.common.v1alpha1.Operation` representations and
    provides helpers to:

    - inspect operation metadata (id, resource_id, timestamps),
    - poll/update the operation state via the corresponding operation
      service, and
    - wait for completion either asynchronously or synchronously.

    The wrapper stores an operation service client bound to a
    :class:`nebius.aio.constant_channel.Constant` that points at the provided
    ``source_method`` and reuses the provided ``channel`` for network/auth
    behaviors.

    :param source_method: the originating ``service.method`` name used to build a
        constant channel for operation management calls
    :param channel: channel used for network and auth operations
    :type channel: :class:`ClientChannelInterface`
    :param operation: an operation protobuf instance (v1 or v1alpha1)
    :type operation: either :class:`nebius.api.nebius.common.v1.Operation` or
        :class:`nebius.api.nebius.common.v1alpha1.Operation`, or their protobuf
        classes.

    Example
    -------

    Operation from a service action (e.g., creating a bucket)::

        from nebius.sdk import SDK
        from nebius.aio.cli_config import Config
        from nebius.api.nebius.storage.v1 import (
            BucketServiceClient,
            CreateBucketRequest
        )

        sdk = SDK(config_reader=Config())
        service = BucketServiceClient(sdk)

        # Create operation from service action
        operation = await service.create(CreateBucketRequest(
            # fill-in necessary fields
        ))

        # Wait for completion
        await operation.wait()
        print(f"New bucket ID: {operation.resource_id}")

    Operation from list of operations::

        from nebius.sdk import SDK
        from nebius.aio.cli_config import Config
        from nebius.api.nebius.storage.v1 import BucketServiceClient
        from nebius.api.nebius.common.v1 import ListOperationsRequest

        sdk = SDK(config_reader=Config())
        service = BucketServiceClient(sdk)

        # Get operation service client from the bucket service
        operation_service = service.operation_service()
        operations_response = await operation_service.list(ListOperationsRequest(
            # fill-in necessary fields
        ))

        # Get first operation from list
        if operations_response.operations:
            operation = operations_response.operations[0]

            # Manual update
            await operation.update()
            print(f"Operation status: {operation.status()}")
    """

    def __init__(
        self,
        source_method: str,
        channel: ClientChannelInterface,
        operation: OperationPb,
    ) -> None:
        """Create an operation wrapper from the operation protobuf."""
        from nebius.api.nebius.common.v1 import (
            GetOperationRequest,
            Operation,
            OperationServiceClient,
        )
        from nebius.api.nebius.common.v1alpha1 import (
            GetOperationRequest as OldGet,
        )
        from nebius.api.nebius.common.v1alpha1 import (
            Operation as Old,
        )
        from nebius.api.nebius.common.v1alpha1 import (
            OperationServiceClient as OldClient,
        )

        self._channel = channel
        _operation: OperationPb | Operation | Old = operation
        if isinstance(_operation, Operation.__PB2_CLASS__):
            _operation = Operation(_operation)
        if isinstance(_operation, Old.__PB2_CLASS__):
            _operation = Old(_operation)

        if isinstance(_operation, Operation):
            self._service: OperationServiceClient | OldClient = OperationServiceClient(
                Constant(source_method, channel)
            )
            self._get_request_obj: type[GetOperationRequest | OldGet] = (
                GetOperationRequest
            )
        elif isinstance(_operation, Old):
            self._service = OldClient(Constant(source_method, channel))
            self._get_request_obj = OldGet
        else:
            raise SDKError(f"Operation type {type(_operation)} not supported.")

        self._operation: Operation | Old = _operation

    def __repr__(self) -> str:
        """Return a compact string representation useful for debugging."""
        return (
            f"Operation({self.id}, resource_id: {self.resource_id}, "
            f"status: {self.status()})"
        )

    def status(self) -> RequestStatus | None:
        """Return the operation's current status object or ``None``.

        :rtype: :class:`RequestStatus` or nothing
        """
        return self._operation.status

    def done(self) -> bool:
        """Return True when the operation has reached a terminal state."""
        return self.status() is not None

    async def update(
        self,
        **kwargs: Unpack[RequestKwargs],
    ) -> None:
        """Fetch the latest operation data from the operation service.

        This coroutine performs a single get operation using the internal
        operation service client and replaces the wrapped operation object
        with the returned value.

        :param kwargs: additional request keyword arguments
            see :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        """
        if self.done():
            return

        req = self._service.get(
            self._get_request_obj(id=self.id),  # type: ignore
            **kwargs,
        )
        new_op = await req
        self._set_new_operation(new_op._operation)  # type: ignore

    def sync_wait(
        self,
        interval: float | timedelta = 1,
        timeout: float | None = None,
        poll_iteration_timeout: float | None | UnsetType = Unset,
        poll_per_retry_timeout: float | None | UnsetType = Unset,
        poll_retries: int | None = None,
        **kwargs: Unpack[RequestKwargsForOperation],
    ) -> None:
        """Synchronously wait for the operation to complete.

        This helper wraps :meth:`wait` and executes it in the channel's
        synchronous runner so callers that are not coroutine-based can wait
        for operation completion.

        See :meth:`wait` for parameter details.
        """
        run_timeout = None if timeout is None else timeout + 0.2
        return self._channel.run_sync(
            self.wait(
                interval=interval,
                timeout=timeout,
                poll_iteration_timeout=poll_iteration_timeout,
                poll_per_retry_timeout=poll_per_retry_timeout,
                poll_retries=poll_retries,
                **kwargs,
            ),
            run_timeout,
        )

    def sync_update(
        self,
        **kwargs: Unpack[RequestKwargs],
    ) -> None:
        """Synchronously perform a single update of the operation state.

        This wraps the coroutine :meth:`update` and runs it via the channel's
        synchronous runner. A small safety margin is added to the provided
        timeout to allow for scheduling overhead.

        :param kwargs: additional request keyword arguments
            see :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        """
        timeout = kwargs.get("timeout", Unset)
        run_timeout: float | None = None
        if isinstance(timeout, (int, float)):
            run_timeout = timeout + 0.2
        elif isinstance(timeout, UnsetType):
            run_timeout = DEFAULT_TIMEOUT + 0.2
        return self._channel.run_sync(
            self.update(
                **kwargs,
            ),
            run_timeout,
        )

    async def wait(
        self,
        interval: float | timedelta = 1,
        timeout: float | None = None,
        poll_iteration_timeout: float | UnsetType | None = Unset,
        poll_per_retry_timeout: float | UnsetType | None = Unset,
        poll_retries: int | None = None,
        **kwargs: Unpack[RequestKwargsForOperation],
    ) -> None:
        """Asynchronously wait until the operation reaches a terminal state.

        The method repeatedly invokes :meth:`update` at the specified
        ``interval`` until the operation is done or the overall ``timeout`` is
        reached. Certain transient errors (deadline exceeded) are treated as
        ignorable and will be retried.

        :param interval: polling interval (seconds or timedelta)
        :type interval: `float` or `timedelta`
        :param timeout: overall timeout (seconds) for waiting, or `None` for
            infinite timeout, default infinite.
        :type timeout: optional `float`
        :param poll_iteration_timeout: timeout used for each polling iteration, will be
            passed as the ``timeout`` to each :meth:`update` call.
        :type poll_iteration_timeout: optional `float` or `None`
        :param poll_per_retry_timeout: per-retry timeout for polling requests, will
            be passed as the ``per_retry_timeout`` to each :meth:`update` call.
        :type poll_per_retry_timeout: optional `float` or `None`, will be passed as the
            ``per_retry_timeout`` to each :meth:`update` call.
        :param poll_retries: retry count used for polling requests, will be passed as
            the ``retries`` to each :meth:`update` call.
        :param kwargs: additional request keyword arguments
            see :class:`nebius.aio.request_kwargs.RequestKwargsForOperation` for
            details.

        :raises TimeoutError: when the overall timeout is exceeded
        """

        start = time()
        if poll_iteration_timeout is None:
            if timeout is not None:
                poll_iteration_timeout = min(5, timeout)
        if isinstance(interval, timedelta):
            interval = interval.total_seconds()
        from nebius.aio.service_error import RequestError as ServiceRequestError

        def _is_ignorable(err: Exception) -> bool:
            # TimeoutError raised locally or RequestError with DEADLINE_EXCEEDED
            if isinstance(err, TimeoutError):
                return True
            if isinstance(err, ServiceRequestError):
                try:
                    return err.status.code == StatusCode.DEADLINE_EXCEEDED
                except Exception:  # pragma: no cover - defensive
                    return False
            return False

        async def _safe_update() -> None:
            try:
                await self.update(
                    timeout=poll_iteration_timeout,
                    per_retry_timeout=poll_per_retry_timeout,
                    retries=poll_retries,
                    **kwargs,
                )
            except Exception as e:  # noqa: S110
                if not _is_ignorable(e):
                    raise

        if not self.done():
            await _safe_update()
        while not self.done():
            current_time = time()
            if timeout is not None and current_time > timeout + start:
                raise TimeoutError("Operation wait timeout")
            await sleep(interval)
            await _safe_update()

    def _set_new_operation(self, operation: OperationPb) -> None:
        """Replace the wrapped operation object with a new instance.

        The replacement is only allowed when the new operation has the same
        protobuf class as the currently wrapped object; otherwise an
        :class:`SDKError` is raised.
        """
        if isinstance(operation, self._operation.__class__):
            self._operation = operation  # type: ignore
        else:
            raise SDKError(f"Operation type {type(operation)} not supported.")

    @property
    def id(self) -> str:
        """Return the operation identifier (string)."""
        return self._operation.id

    @property
    def description(self) -> str:
        """Return the operation description as provided by the service."""
        return self._operation.description

    @property
    def created_at(self) -> datetime:
        """Return the operation creation timestamp.

        If the underlying protobuf does not expose a creation time this helper
        returns the current time in the local timezone.
        :rtype: datetime
        """
        ca = self._operation.created_at
        if ca is None:  # type: ignore[unused-ignore]
            return datetime.now(local_timezone)
        return ca

    @property
    def created_by(self) -> str:
        """Return the identity that created the operation (string)."""
        return self._operation.created_by

    @property
    def finished_at(self) -> datetime | None:
        """Return the completion timestamp for the operation or ``None`` if
        the operation hasn't finished yet.
        """
        return self._operation.finished_at

    @property
    def resource_id(self) -> str:
        """Return the resource id associated with the operation."""
        return self._operation.resource_id

    def successful(self) -> bool:
        """Return True when the operation completed successfully."""
        s = self.status()
        return s is not None and s.code == StatusCode.OK

    def raw(self) -> OperationPb:
        """Return the underlying operation protobuf object.

        Use this to access version-specific fields that are not exposed by the
        normalized wrapper.
        """
        return self._operation  # type: ignore



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/postgresql/v1alpha1/backup.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+nebius/msp/postgresql/v1alpha1/backup.proto\x12\x1enebius.msp.postgresql.v1alpha1\x1a\x1fgoogle/protobuf/timestamp.proto\"\xee\x02\n\x06\x42\x61\x63kup\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12*\n\x11source_cluster_id\x18\x02 \x01(\tR\x0fsourceClusterId\x12\x41\n\x0e\x63reation_start\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rcreationStart\x12\x43\n\x0f\x63reation_finish\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x63reationFinish\x12.\n\x13source_cluster_name\x18\x05 \x01(\tR\x11sourceClusterName\x12\x34\n\x16source_cluster_visible\x18\x06 \x01(\x08R\x14sourceClusterVisible\x12\x1b\n\ton_demand\x18\x07 \x01(\x08R\x08onDemand\x12\x1d\n\nsize_bytes\x18\x08 \x01(\x03R\tsizeBytesBt\n%ai.nebius.pub.msp.postgresql.v1alpha1B\x0b\x42\x61\x63kupProtoP\x01Z<github.com/nebius/gosdk/proto/nebius/msp/postgresql/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.postgresql.v1alpha1.backup_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%ai.nebius.pub.msp.postgresql.v1alpha1B\013BackupProtoP\001Z<github.com/nebius/gosdk/proto/nebius/msp/postgresql/v1alpha1'
  _globals['_BACKUP']._serialized_start=113
  _globals['_BACKUP']._serialized_end=479
# @@protoc_insertion_point(module_scope)
